@@ -1,8 +1,8 @@
 // L2TPNS Command Line Interface
 // vim: sw=8 ts=8
 
-char const *cvs_name = "$Name: release_2_0_14 $";
-char const *cvs_id_cli = "$Id: cli.c,v 1.43 2004/12/17 00:28:00 bodea Exp $";
+char const *cvs_name = "$Name: release_2_0_22 $";
+char const *cvs_id_cli = "$Id: cli.c,v 1.43.2.6 2005/05/30 02:55:40 bodea Exp $";
 
 #include <stdio.h>
 #include <stdarg.h>
@@ -289,8 +289,9 @@ void cli_do(int sockfd)
 	if (fork_and_close()) return;
 	if (getpeername(sockfd, (struct sockaddr *)&addr, &l) == 0)
 	{
-		LOG(3, 0, 0, "Accepted connection to CLI from %s\n", fmtaddr(addr.sin_addr.s_addr, 0));
 		require_auth = addr.sin_addr.s_addr != inet_addr("127.0.0.1");
+		LOG(require_auth ? 3 : 4, 0, 0, "Accepted connection to CLI from %s\n",
+			fmtaddr(addr.sin_addr.s_addr, 0));
 	}
 	else
 		LOG(0, 0, 0, "getpeername() failed on cli socket.  Requiring authentication: %s\n", strerror(errno));
@@ -321,7 +322,9 @@ void cli_do(int sockfd)
 	cli_loop(cli, sockfd);
 
 	close(sockfd);
-	LOG(3, 0, 0, "Closed CLI connection from %s\n", fmtaddr(addr.sin_addr.s_addr, 0));
+	LOG(require_auth ? 3 : 4, 0, 0, "Closed CLI connection from %s\n",
+		fmtaddr(addr.sin_addr.s_addr, 0));
+
 	exit(0);
 }
 
@@ -403,6 +406,7 @@ static int cmd_show_session(struct cli_def *cli, char *command, char **argv, int
 			cli_print(cli, "\tTunnel ID:\t%d", session[s].tunnel);
 			cli_print(cli, "\tIP address:\t%s", fmtaddr(htonl(session[s].ip), 0));
 			cli_print(cli, "\tUnique SID:\t%lu", session[s].unique_id);
+			cli_print(cli, "\tOpened:\t\t%u seconds", abs(time_now - session[s].opened));
 			cli_print(cli, "\tIdle time:\t%u seconds", abs(time_now - session[s].last_packet));
 			cli_print(cli, "\tNext Recv:\t%u", session[s].nr);
 			cli_print(cli, "\tNext Send:\t%u", session[s].ns);
@@ -645,23 +649,24 @@ static int cmd_show_counters(struct cli_def *cli, char *command, char **argv, in
 	if (CLI_HELP_REQUESTED)
 		return CLI_HELP_NO_ARGS;
 
-	cli_print(cli, "%-10s %-8s %-10s %-8s", "Ethernet", "Bytes", "Packets", "Errors");
-	cli_print(cli, "%-10s %8u %8u %8u", "RX",
+	cli_print(cli, "%-10s %10s %10s %10s %10s", "Ethernet", "Bytes", "Packets", "Errors", "Dropped");
+	cli_print(cli, "%-10s %10u %10u %10u %10u", "RX",
 			GET_STAT(tun_rx_bytes),
 			GET_STAT(tun_rx_packets),
-			GET_STAT(tun_rx_errors));
-	cli_print(cli, "%-10s %8u %8u %8u", "TX",
+			GET_STAT(tun_rx_errors),
+			GET_STAT(tun_rx_dropped));
+	cli_print(cli, "%-10s %10u %10u %10u", "TX",
 			GET_STAT(tun_tx_bytes),
 			GET_STAT(tun_tx_packets),
 			GET_STAT(tun_tx_errors));
 	cli_print(cli, "");
 
-	cli_print(cli, "%-10s %-8s %-10s %-8s %-8s", "Tunnel", "Bytes", "Packets", "Errors", "Retries");
-	cli_print(cli, "%-10s %8u %8u %8u", "RX",
+	cli_print(cli, "%-10s %10s %10s %10s %10s", "Tunnel", "Bytes", "Packets", "Errors", "Retries");
+	cli_print(cli, "%-10s %10u %10u %10u", "RX",
 			GET_STAT(tunnel_rx_bytes),
 			GET_STAT(tunnel_rx_packets),
 			GET_STAT(tunnel_rx_errors));
-	cli_print(cli, "%-10s %8u %8u %8u %8u", "TX",
+	cli_print(cli, "%-10s %10u %10u %10u %10u", "TX",
 			GET_STAT(tunnel_tx_bytes),
 			GET_STAT(tunnel_tx_packets),
 			GET_STAT(tunnel_tx_errors),
@@ -720,6 +725,17 @@ static int cmd_show_counters(struct cli_def *cli, char *command, char **argv, in
 	cli_print(cli, "%-30s%u", "call_radiussend",		GET_STAT(call_radiussend));
 	cli_print(cli, "%-30s%u", "call_radiusretry",		GET_STAT(call_radiusretry));
 #endif
+
+	{
+		time_t l = GET_STAT(last_reset);
+		char *t = ctime(&l);
+		char *p = strchr(t, '\n');
+		if (p) *p = 0;
+
+		cli_print(cli, "");
+		cli_print(cli, "Last counter reset %s", t);
+	}
+
 	return CLI_OK;
 }
 
@@ -1112,8 +1128,10 @@ static int cmd_clear_counters(struct cli_def *cli, char *command, char **argv, i
 	if (CLI_HELP_REQUESTED)
 		return CLI_HELP_NO_ARGS;
 
-	cli_print(cli, "Counters cleared");
+	memset(_statistics, 0, sizeof(struct Tstats));
 	SET_STAT(last_reset, time(NULL));
+
+	cli_print(cli, "Counters cleared");
 	return CLI_OK;
 }
 
