@@ -1,6 +1,6 @@
 // L2TPNS PPP Stuff
 
-char const *cvs_id_ppp = "$Id: ppp.c,v 1.39 2004/12/16 08:49:53 bodea Exp $";
+char const *cvs_id_ppp = "$Id: ppp.c,v 1.39.2.2 2005/02/14 05:55:49 bodea Exp $";
 
 #include <stdio.h>
 #include <string.h>
@@ -60,13 +60,18 @@ void processpap(tunnelidt t, sessionidt s, uint8_t *p, uint16_t l)
 	{
 		uint8_t *b = p;
 		b += 4;
+		user[0] = pass[0] = 0;
 		if (*b && *b < sizeof(user))
+		{
 			memcpy(user, b + 1, *b);
-		user[*b] = 0;
-		b += 1 + *b;
-		if (*b && *b < sizeof(pass))
-			memcpy(pass, b + 1, *b);
-		pass[*b] = 0;
+			user[*b] = 0;
+			b += 1 + *b;
+			if (*b && *b < sizeof(pass))
+			{
+				memcpy(pass, b + 1, *b);
+				pass[*b] = 0;
+			}
+		}
 		LOG(3, s, t, "PAP login %s/%s\n", user, pass);
 	}
 	if (session[s].ip || !session[s].radius)
@@ -766,7 +771,7 @@ void processipin(tunnelidt t, sessionidt s, uint8_t *p, uint16_t l)
 
 	session[s].cin += l - 4;
 	session[s].total_cin += l - 4;
-	sess_count[s].cin += l - 4;
+	sess_local[s].cin += l - 4;
 
 	session[s].pin++;
 	eth_tx += l - 4;
@@ -801,7 +806,7 @@ void send_ipin(sessionidt s, uint8_t *buf, int len)
 	// Increment packet counters
 	session[s].cin += len - 4;
 	session[s].total_cin += len - 4;
-	sess_count[s].cin += len - 4;
+	sess_local[s].cin += len - 4;
 
 	session[s].pin++;
 	eth_tx += len - 4;
